from itertools import combinations


class Ohms:
    """A class representing basic electrical calculations based on Ohm's Law."""
    
    def volts(self, I: float, R: float) -> float:
        """
        Calculate voltage using Ohm's Law.

        Parameters:
            I (float): The current in amperes.
            R (float): The resistance in ohms.

        Returns:
            float: The voltage in volts.
        """
        V = I * R
        return V
    
    def current(self, V: float, R: float) -> float:
        """
        Calculate current using Ohm's Law.

        Parameters:
            V (float): The voltage in volts.
            R (float): The resistance in ohms.

        Returns:
            float: The current in amperes.
        """
        I = V / R
        return I
        
    def resistance(self, V: float, I: float) -> float:
        """
        Calculate resistance using Ohm's Law.

        Parameters:
            V (float): The voltage in volts.
            I (float): The current in amperes.

        Returns:
            float: The resistance in ohms.
        """
        R = V / I
        return R
    
    def watts(self, I: float, R: float) -> float:
        """
        Calculate electrical power in watts using current and resistance.

        This method uses the formula P = I^2 * R to calculate electrical power,
        where P is the power in watts, I is the current in amperes, and R is the
        resistance in ohms.

        Parameters:
        -----------
        I : float
            Electrical current in amperes (A).
        R : float
            Electrical resistance in ohms (Ω).

        Returns:
        --------
        float
            Electrical power in watts (W).

        Example:
        --------
        >>> calculator = Ohms()
        >>> power = calculator.watts(3.0, 5.0)
        >>> print(power)
        45.0
        """
        W = I**2 * R
        return W
    
    def find_resistor(self, 
                  source: float,
                  component_voltage: float,
                  component_current: float = 0.02 # initial value for LED, change the value for other components
                  ) -> float:
        """
        Calculate the resistor value needed to limit the current flowing through a component using Ohm's Law.

        This method calculates the resistor value needed to limit the current flowing through a component
        to a specified value, considering the source voltage and the forward voltage drop of the component.

        Parameters:
            source (float): The voltage of the power source in volts.
            component_voltage (float): The forward voltage drop of the component in volts.
            component_current (float): The desired current for the component in amperes.

        Returns:
            float: The resistance value needed for the circuit in ohms.
        """
        
        U = source - component_voltage  # Calcula a tensão sobre o resistor
        R = U / component_current       # Calcula a resistência usando a Lei de Ohm
        return R

    
    def series(self, *resistors: float) -> float:
        """
        Calculate the total resistance of resistors in series.

        Parameters:
            resistors (float): Resistances of the resistors in ohms.

        Returns:
            float: The total resistance in ohms.
        """
        return sum(resistors)
    
    def parallel(self, *resistors: float) -> float:
        """
        Calculate the total resistance of resistors in parallel.

        Parameters:
            resistors (float): Resistances of the resistors in ohms.

        Returns:
            float: The total resistance in ohms.
        """
        inverse_total_resistance = sum(1/r for r in resistors)
        return 1 / inverse_total_resistance
    
    def best_combination(self, source_voltage: float, component_voltage: float, component_current: float, resistors: list) -> tuple:
        """
        Find the best combination of resistors to achieve the desired voltage drop across the component.

        Parameters:
            source_voltage (float): The voltage of the power source in volts.
            component_voltage (float): The desired voltage drop across the component in volts.
            component_current (float): The desired current for the component in amperes.
            resistors (list): A list of available resistors in ohms.

        Returns:
            tuple: The best combination of resistors and the resulting voltage across the component.
        """
        best_combination = None
        closest_voltage = float('inf')
        target_voltage = source_voltage - component_voltage

        for r in range(1, len(resistors) + 1):
            for combo in combinations(resistors, r):
                total_resistance = self.series(*combo)
                total_current = source_voltage / (total_resistance + (component_voltage / component_current))
                voltage_drop_across_resistors = total_current * total_resistance
                voltage_across_component = source_voltage - voltage_drop_across_resistors
                if abs(voltage_across_component - component_voltage) < abs(closest_voltage - component_voltage):
                    closest_voltage = voltage_across_component
                    best_combination = combo

        return best_combination, closest_voltage




import argparse
from ohmslaw.__main__ import Ohms

def calculate_volts(I, R):
    """Calculate voltage using Ohm's Law."""
    ohms = Ohms()
    return ohms.volts(I, R)

def calculate_current(V, R):
    """Calculate current using Ohm's Law."""
    ohms = Ohms()
    return ohms.current(V, R)

def calculate_resistance(V, I):
    """Calculate resistance using Ohm's Law."""
    ohms = Ohms()
    return ohms.resistance(V, I)

def calculate_watts(I, R):
    """Calculate Watts."""
    ohms = Ohms()
    return ohms.watts(I, R)

def find_resistor(source, component_voltage, component_current=0.02):
    """Calculate resistor value to limit current."""
    ohms = Ohms()
    return ohms.find_resistor(source, component_voltage, component_current)

def calculate_series(*resistors):
    """Calculate total resistance for resistors in series."""
    ohms = Ohms()
    return ohms.series(*resistors)

def calculate_parallel(*resistors):
    """Calculate total resistance for resistors in parallel."""
    ohms = Ohms()
    return ohms.parallel(*resistors)

def calculate_best_combination(source_voltage, component_voltage, component_current, resistors):
    """Find the best combination of resistors."""
    ohms = Ohms()
    return ohms.best_combination(source_voltage, component_voltage, component_current, resistors)

def main() -> str:
    parser = argparse.ArgumentParser(description="Perform electrical calculations based on Ohm's Law")
    subparsers = parser.add_subparsers(dest='operation', help='Operation to perform')

    # Subparser for volts operation
    volts_parser = subparsers.add_parser('volts', help='Calculate voltage using Ohm\'s Law')
    volts_parser.add_argument('-I', type=float, required=True, help='Current in amperes')
    volts_parser.add_argument('-R', type=float, required=True, help='Resistance in ohms')

    # Subparser for current operation
    current_parser = subparsers.add_parser('current', help='Calculate current using Ohm\'s Law')
    current_parser.add_argument('-V', type=float, required=True, help='Voltage in volts')
    current_parser.add_argument('-R', type=float, required=True, help='Resistance in ohms')

    # Subparser for resistance operation
    resistance_parser = subparsers.add_parser('resistance', help='Calculate resistance using Ohm\'s Law')
    resistance_parser.add_argument('-V', type=float, required=True, help='Voltage in volts')
    resistance_parser.add_argument('-I', type=float, required=True, help='Current in amperes')

    watts_parser = subparsers.add_parser('watts', help='Calculate watts using Ohm\'s Law')
    watts_parser.add_argument('-R', type=float, required=True, help='Resistance in ohms')
    watts_parser.add_argument('-I', type=float, required=True, help='Current in amperes')

    # Subparser for find_resistor operation
    find_resistor_parser = subparsers.add_parser('find_resistor', help='Calculate resistor value to limit current')
    find_resistor_parser.add_argument('--source', type=float, required=True, help='Source voltage in volts')
    find_resistor_parser.add_argument('--component_voltage', type=float, required=True, help='Component voltage in volts')
    find_resistor_parser.add_argument('--component_current', type=float, default=0.02, help='Component current in amperes (default: 0.02)')

    # Subparser for series operation
    series_parser = subparsers.add_parser('series', help='Calculate total resistance for resistors in series')
    series_parser.add_argument('resistors', type=float, nargs='+', help='List of resistances in ohms')

    # Subparser for parallel operation
    parallel_parser = subparsers.add_parser('parallel', help='Calculate total resistance for resistors in parallel')
    parallel_parser.add_argument('resistors', type=float, nargs='+', help='List of resistances in ohms')

    # Subparser for best_combination operation
    best_combination_parser = subparsers.add_parser('best_combination', help='Find the best combination of resistors')
    best_combination_parser.add_argument('-sv', '--source_voltage', type=float, required=True, help='Source voltage in volts')
    best_combination_parser.add_argument('-cv', '--component_voltage', type=float, required=True, help='Component voltage in volts')
    best_combination_parser.add_argument('-cc', '--component_current', type=float, required=True, help='Component current in amperes')
    best_combination_parser.add_argument('-r', '--resistors', type=float, nargs='+', required=True, help='List of available resistors in ohms')

    args = parser.parse_args()

    result = None

    if args.operation == 'volts':
        result = str(calculate_volts(args.I, args.R)) + " V"
    elif args.operation == 'current':
        result = str(calculate_current(args.V, args.R)) + " A"
    elif args.operation == 'resistance':
        result = str(calculate_resistance(args.V, args.I)) + " Ohms"
    elif args.operation == 'watts':
        result = str(calculate_watts(args.R, args.I)) + " Watts"
    elif args.operation == 'find_resistor':
        result = str(find_resistor(args.source, args.component_voltage, args.component_current)) + " Ohms"
    elif args.operation == 'series':
        result = str(calculate_series(*args.resistors)) + " Ohms"
    elif args.operation == 'parallel':
        result = str(calculate_parallel(*args.resistors)) + " Ohms"
    elif args.operation == 'best_combination':
        combination, voltage = calculate_best_combination(args.source_voltage, args.component_voltage, args.component_current, args.resistors)
        result = f"Best combination: {combination} with voltage: {voltage} V"

    return result

if __name__ == "__main__":
    main()
